import { defineComponent, toRefs, reactive, ref, computed, watchEffect, h, mergeProps, useModel } from 'vue';
import { reactiveOmit, useListeners, usePrevious, useSize, useForwardRef, useComposedRefs, useControllable } from '@oku-ui/use-composable';
import { composeEventHandlers } from '@oku-ui/utils';
import { primitiveProps, Primitive } from '@oku-ui/primitive';
import { ScopePropObject, createProvideScope } from '@oku-ui/provide';
import { OkuPresence } from '@oku-ui/presence';

function isIndeterminate(checked) {
  return checked === "indeterminate";
}
function getState(checked) {
  return isIndeterminate(checked) ? "indeterminate" : checked ? "checked" : "unchecked";
}

const scopeCheckboxProps = {
  scopeOkuCheckbox: {
    ...ScopePropObject
  }
};
const CHECKBOX_NAME = "OkuCheckbox";
const CHECKBOX_INDICATOR_NAME = "OkuCheckboxIndicator";
const CHECKBOX_BUBBLE_INPUT_NAME = "OkuBubbleInput";
const [createCheckboxProvider, createCheckboxScope] = createProvideScope(CHECKBOX_NAME);
const [checkboxProvider, useCheckboxInject] = createCheckboxProvider(CHECKBOX_NAME);
const checkboxProps = {
  props: {
    modelValue: {
      type: [Boolean, String],
      default: void 0
    },
    checked: {
      type: [Boolean, String],
      default: void 0
    },
    defaultChecked: {
      type: [Boolean, String],
      default: void 0
    },
    required: {
      type: Boolean
    },
    name: {
      type: String
    },
    disabled: {
      type: [Boolean, String]
    },
    value: {
      type: String,
      default: "on"
    },
    ...primitiveProps
  },
  emits: {
    // eslint-disable-next-line unused-imports/no-unused-vars
    "update:modelValue": (checked) => true,
    // eslint-disable-next-line unused-imports/no-unused-vars
    "checkedChange": (checked) => true,
    // eslint-disable-next-line unused-imports/no-unused-vars
    "keydown": (event) => true,
    // eslint-disable-next-line unused-imports/no-unused-vars
    "click": (event) => true
  }
};
const checkboxIndicatorProps = {
  props: {
    forceMount: {
      type: Boolean,
      default: void 0
    },
    ...primitiveProps
  },
  emits: {}
};
const bubbleInputProps = {
  props: {
    checked: {
      type: [Boolean, String],
      required: true
    },
    control: {
      type: [HTMLElement, null],
      default: null,
      required: true
    },
    bubbles: {
      type: Boolean,
      default: true,
      required: true
    }
  },
  emits: {}
};

const bubbleInput = defineComponent({
  name: CHECKBOX_BUBBLE_INPUT_NAME,
  inheritAttrs: false,
  props: {
    ...bubbleInputProps.props
  },
  emits: {
    ...bubbleInputProps.emits
  },
  setup(props, { attrs }) {
    const {
      checked,
      bubbles,
      control,
      ...inputProps
    } = toRefs(props);
    const _reactive = reactive(inputProps);
    const otherProps = reactiveOmit(_reactive, (key, _value) => key === void 0);
    const emits = useListeners();
    const inputRef = ref(null);
    const prevChecked = usePrevious(checked);
    const controlSize = computed(() => useSize(control));
    watchEffect(() => {
      const input = inputRef.value;
      const inputProto = window.HTMLInputElement.prototype;
      const descriptor = Object.getOwnPropertyDescriptor(inputProto, "checked");
      const setChecked = descriptor.set;
      if (prevChecked.value !== checked.value && setChecked) {
        const event = new Event("click", { bubbles: bubbles.value });
        input.indeterminate = isIndeterminate(checked.value);
        setChecked.call(input, isIndeterminate(checked.value) ? false : checked);
        input.dispatchEvent(event);
      }
    });
    return () => h("input", {
      "type": "checkbox",
      "aria-hidden": true,
      "defaultChecked": isIndeterminate(checked.value) ? false : checked.value,
      ...mergeProps(attrs, otherProps, emits),
      "tabIndex": -1,
      "ref": inputRef,
      "style": {
        ...attrs.style,
        ...controlSize.value,
        position: "absolute",
        pointerEvents: "none",
        opacity: 0,
        margin: "0px"
      }
    });
  }
});
const OkuBubbleInput = bubbleInput;

const Checkbox = defineComponent({
  name: CHECKBOX_NAME,
  components: {
    OkuBubbleInput
  },
  inheritAttrs: false,
  props: {
    ...checkboxProps.props,
    ...scopeCheckboxProps
  },
  emits: checkboxProps.emits,
  setup(props, { attrs, emit, slots }) {
    const {
      scopeOkuCheckbox,
      name,
      checked: checkedProp,
      defaultChecked,
      required,
      disabled,
      value,
      ...checkboxProps2
    } = toRefs(props);
    const _reactive = reactive(checkboxProps2);
    const otherProps = reactiveOmit(_reactive, (key, _value) => key === void 0);
    const forwardedRef = useForwardRef();
    const emits = useListeners();
    const buttonRef = ref(null);
    const composedRefs = useComposedRefs(forwardedRef, buttonRef);
    const hasConsumerStoppedPropagationRef = ref(false);
    const isFormControl = computed(() => buttonRef.value instanceof HTMLElement ? Boolean(buttonRef.value.closest("form")) : false);
    const modelValue = useModel(props, "modelValue");
    const proxyChecked = computed({
      get: () => modelValue.value !== void 0 ? modelValue.value : checkedProp.value !== void 0 ? checkedProp.value : void 0,
      set: () => {
      }
    });
    const { state, updateValue } = useControllable({
      prop: computed(() => proxyChecked.value),
      defaultProp: computed(() => defaultChecked.value),
      onChange: (result) => {
        modelValue.value = result;
        emit("checkedChange", result);
        emit("update:modelValue", result);
      },
      initialValue: false
    });
    const initialCheckedStateRef = ref(state.value);
    watchEffect((onInvalidate) => {
      const form = buttonRef.value?.form;
      if (form) {
        const reset = () => updateValue(initialCheckedStateRef.value);
        form.addEventListener("reset", reset);
        onInvalidate(() => form.removeEventListener("reset", reset));
      }
    });
    checkboxProvider({
      scope: scopeOkuCheckbox.value,
      state,
      disabled
    });
    return () => [
      h(Primitive.button, {
        "type": "button",
        "role": "checkbox",
        "aria-checked": computed(() => isIndeterminate(state.value) ? "mixed" : state.value).value,
        "aria-required": required.value,
        "data-state": computed(() => getState(state.value)).value,
        "data-disabled": disabled.value ? "" : void 0,
        "disabled": disabled.value,
        "value": value.value,
        ...mergeProps(attrs, otherProps, emits),
        "ref": composedRefs,
        "onKeydown": composeEventHandlers((event) => {
          emit("keydown", event);
        }, (event) => {
          if (event.key === "Enter")
            event.preventDefault();
        }),
        "onClick": composeEventHandlers((event) => {
          emit("click", event);
        }, (event) => {
          updateValue(isIndeterminate(state.value) ? true : !state.value);
          if (isFormControl.value) {
            if (!hasConsumerStoppedPropagationRef.value)
              event.stopPropagation();
          }
        })
      }, () => slots.default?.()),
      isFormControl.value && h(OkuBubbleInput, {
        control: buttonRef.value,
        bubbles: computed(() => !hasConsumerStoppedPropagationRef.value).value,
        name: name.value,
        value: value.value,
        checked: state.value,
        required: required.value,
        disabled: disabled.value,
        // We transform because the input is absolutely positioned but we have
        // rendered it **after** the button. This pulls it back to sit on top
        // of the button.
        style: { transform: "translateX(-100%)" }
      })
    ];
  }
});
const OkuCheckbox = Checkbox;

const checkboxIndicator = defineComponent({
  name: CHECKBOX_INDICATOR_NAME,
  props: {
    ...checkboxIndicatorProps.props,
    ...scopeCheckboxProps
  },
  emits: {
    ...checkboxIndicatorProps.emits
  },
  setup(props, { attrs, slots }) {
    const {
      scopeOkuCheckbox,
      forceMount,
      ...indicatorProps
    } = toRefs(props);
    const _reactive = reactive(indicatorProps);
    const otherProps = reactiveOmit(_reactive, (key, _value) => key === void 0);
    const forwardedRef = useForwardRef();
    const emits = useListeners();
    const inject = useCheckboxInject(CHECKBOX_INDICATOR_NAME, scopeOkuCheckbox.value);
    return () => h(OkuPresence, {
      present: computed(() => forceMount.value || isIndeterminate(inject.state.value) || inject.state.value === true).value
    }, () => h(Primitive.span, {
      "data-state": computed(() => getState(inject.state.value)).value,
      "data-disabled": inject.disabled?.value ? "" : void 0,
      ...mergeProps(attrs, otherProps, emits),
      "ref": forwardedRef,
      "style": { pointerEvents: "none", ...attrs.style }
    }, () => slots.default?.()));
  }
});
const OkuCheckboxIndicator = checkboxIndicator;

export { OkuCheckbox, OkuCheckboxIndicator };
